<?php

defined ('_JEXEC') or die('Restricted access');

/**
 * Shipment plugin for general, rules-based shipments, like regular postal services with complex shipping cost structures
 * Advanced part, implementing general mathematical expression evaluation
 *
 * @package VirtueMart
 * @subpackage Plugins - shipment.
 * @copyright Copyright (C) 2013 Reinhold Kainhofer, office@open-tools.net
 * @license http://www.gnu.org/copyleft/gpl.html GNU/GPL, see LICENSE.txt
 * VirtueMart is free software. This version may have been modified pursuant
 * to the GNU General Public License, and as distributed it includes or
 * is derivative of works licensed under the GNU General Public License or
 * other free or open source software licenses.
 *
 * @author Reinhold Kainhofer, based on the weight_countries shipping plugin by Valerie Isaksen
 *
 */

if (!class_exists ('vmPSPlugin')) {
	require(JPATH_VM_PLUGINS . DS . 'vmpsplugin.php');
}
if (!class_exists ('plgVmShipmentRules_Shipping_Base')) {
	require (dirname(__FILE__).DS.'rules_shipping_base.php');
}


/** Shipping costs according to general rules.
 *  Derived from the standard plugin, no need to change anything! The standard plugin already uses the advanced rules class defined below, if it can be found
 */
class plgVmShipmentRules_Shipping_Advanced extends plgVmShipmentRules_Shipping_Base {
	function __construct (& $subject, $config) {
		parent::__construct ($subject, $config);
		$this->helper->registerCallback('initRule',				array($this, 'initRule'));
		$this->helper->registerCallback('addCustomCartValues',	array($this, 'addCustomCartValues'));
	}
	public function initRule ($framework, $rulestring, $countries, $ruleinfo) {
		return new ShippingRule_Advanced ($framework, $rulestring, $countries, $ruleinfo);
	}
	/** Allow child classes to add additional variables for the rules 
	 */
	public function addCustomCartValues ($cart, $products, $method, &$values) {
		$values['coupon'] = $cart->couponCode;

		// Postal code Check for UK postal codes: Use regexp to determine if ZIP structure matches and also to extract the parts.
		// Also handle UK overseas areas/islands that use four-letter outward codes rather than "A{1,2}0{1,2}A{0,1} 0AA"
		if (isset($values['zip'])) {
			$zip=strtoupper($values['zip']);
		}
		if (isset($zip) and preg_match('/^\s*(([A-Z]{1,2})(\d{1,2})([A-Z]?)|[A-Z]{4}|GIR)\s*(\d[A-Z]{2})\s*$/', $zip, $match)) {
			$values['uk_outward'] = $match[1];
			$values['uk_area'] = $match[2];
			$values['uk_district'] = $match[3];
			$values['uk_subdistrict'] = $match[4];
			$values['uk_inward'] = $match[5];
		} else {
			$values['uk_outward'] = NULL;
			$values['uk_area'] = NULL;
			$values['uk_district'] = NULL;
			$values['uk_subdistrict'] = NULL;
			$values['uk_inward'] = NULL;
		}
		// Postal code Check for Canadian postal codes: Use regexp to determine if ZIP structure matches and also to extract the parts.
		if (isset($zip) and preg_match('/^\s*(([A-Za-z])(\d)([A-Za-z]))\s*(\d[A-Za-z]\d)\s*$/', $zip, $match)) {
			$values['canada_fsa'] = $match[1];
			$values['canada_area'] = $match[2];
			$values['canada_urban'] = $match[3];
			$values['canada_subarea'] = $match[4];
			$values['canada_ldu'] = $match[5];
		} else {
			$values['canada_fsa'] = NULL;
			$values['canada_area'] = NULL;
			$values['canada_urban'] = NULL;
			$values['canada_subarea'] = NULL;
			$values['canada_ldu'] = NULL;
		}
		// print("<pre>values: ".print_r($values,1)."</pre>");
	}

}
